@@ -1,8 +1,8 @@
 # $File: //member/autrijus/MasonX-Profiler/lib/MasonX/Profiler.pm $ $Author: autrijus $
-# $Revision: #1 $ $Change: 8431 $ $DateTime: 2003/10/16 11:21:57 $
+# $Revision: #5 $ $Change: 8452 $ $DateTime: 2003/10/17 10:52:28 $
 
 package MasonX::Profiler;
-$MasonX::Profiler::VERSION = '0.01';
+$MasonX::Profiler::VERSION = '0.02';
 
 use strict;
 use Time::HiRes qw( time );
@@ -11,13 +11,26 @@ use Time::HiRes qw( time );
 
 MasonX::Profiler - Mason per-component profiler
 
+=head1 VERSION
+
+This document describes version 0.02 of MasonX::Profiler, released
+October 17, 2003.
+
 =head1 SYNOPSIS
 
+In the Mason handler:
+
     use MasonX::Profiler;
     my $ah = HTML::Mason::ApacheHandler->new(
-	preamble => 'MasonX::Profiler->init(my($p), $m, $r);',
+	preamble => 'my $p = MasonX::Profiler->new($m, $r);',
+	# ...
     );
 
+Alternatively, in F<httpd.conf> with L<HTML::Mason::ApacheHandler>:
+
+    PerlModule MasonX::Profiler
+    PerlSetVar MasonPreamble "my $p = MasonX::Profiler->new($m, $r);"
+
 =head1 DESCRIPTION
 
 This module prints per-component profiling information to STDERR (usually
@@ -35,43 +48,53 @@ directed to the Apache error log).  Its output looks like this:
     =Mason= 210.85.16.204 -         /Elements/Callback 0.0294
     =Mason= 210.85.16.204 -     /Elements/Footer 0.0308
     =Mason= 210.85.16.204 - /autohandler 2.9179
+    =Mason= 210.85.16.204 - /Foundry/Home/MyRequests.html ENDS
 
 Each row contains five whitespace-separated fields: C<=Mason=>, remote IP
 address, C<->, indented component name, and the time spent processing that
-component (inclusive).  The initial request is represented by the special
-time field C<BEGINS>.
+component (inclusive).  The beginning and end of the initial request is
+represented by the special time fields C<BEGINS> and C<ENDS>.
 
 =cut
 
-my $depth = 0;
+my %Depth;
 
 sub init {
     my ($class, $p, $m, $r) = @_;
-    my $ip = eval { $r->connection->get_remote_host(
-	Apache::REMOTE_NAME(), $r->per_dir_config
-    ) } or return;
-    $_[1] = $class->new( $m->current_comp->path, $r->uri, $ip );
+    $_[1] = $class->new($m, $r);
 }
 
 sub new {
-    my ($class, $tag, $uri, $ip) = @_;
+    my ($class, $m, $r) = @_;
 
-    return if $tag eq '/l';
+    my $self = {
+	start	=> time(),
+	uri	=> $r->uri,
+	tag	=> $m->current_comp->path,
+	ip	=> (
+	    eval { $r->connection->get_remote_host(
+		Apache::REMOTE_NAME(), $r->per_dir_config,
+	    ) } || '*'
+	),
+    };
 
-    my $self = bless({}, $class);
+    return if $self->{tag} eq '/l';
 
-    $self->{tag}   = $tag;
-    $self->{start} = time;
-    $self->{ip}    = $ip;
+    print STDERR "=Mason= $self->{ip} - $self->{uri} BEGINS\n"
+	unless $Depth{$self->{ip}}{$self->{uri}}++;
 
-    print STDERR "=Mason= $ip - $uri BEGINS\n" unless $depth++;
-    return $self;
+    bless($self, $class);
 }
 
 sub DESTROY {
     my $self = shift;
-    printf STDERR "=Mason= $self->{ip} - " . (' ' x (--$depth*4)) . "%s %.4f\n",
-	$self->{tag}, time - $self->{start};
+    my $indent = ' ' x (4 * --$Depth{$self->{ip}}{$self->{uri}});
+
+    printf STDERR "=Mason= $self->{ip} - $indent" .
+		  "$self->{tag} %.4f\n", (time - $self->{start});
+
+    return if $indent;
+    print STDERR "=Mason= $self->{ip} - $self->{uri} ENDS\n";
 }
 
 1;
